@@ -27,22 +27,28 @@
 #include <glade/glade.h>
 #include <glib/gi18n.h>
 
-#include <libtelepathy/tp-helpers.h>
-#include <libmissioncontrol/mission-control.h>
-
 #include <libempathy/gossip-conf.h>
 #include <libempathy/gossip-contact.h>
 #include <libempathy/gossip-debug.h>
+#include <libempathy/gossip-utils.h>
+#include <libempathy/gossip-chatroom-manager.h>
+#include <libempathy/gossip-chatroom.h>
+#include <libempathy/empathy-contact-list.h>
+#include <libempathy/empathy-contact-manager.h>
 
 #include "empathy-main-window.h"
 #include "ephy-spinner.h"
-#include "gossip-contact-list.h"
+#include "gossip-contact-list-store.h"
+#include "gossip-contact-list-view.h"
 #include "gossip-presence-chooser.h"
 #include "gossip-ui-utils.h"
 #include "gossip-status-presets.h"
 #include "gossip-geometry.h"
 #include "gossip-preferences.h"
 #include "gossip-accounts-dialog.h"
+#include "gossip-about-dialog.h"
+#include "gossip-new-chatroom-dialog.h"
+#include "gossip-chatrooms-window.h"
 
 #define DEBUG_DOMAIN "MainWindow"
 
@@ -52,111 +58,125 @@
 /* Accels (menu shortcuts) can be configured and saved */
 #define ACCELS_FILENAME "accels.txt"
 
-/* Flashing delay for icons (milliseconds). */
-#define FLASH_TIMEOUT 500
-
 /* Name in the geometry file */
 #define GEOMETRY_NAME "main-window"
 
 typedef struct {
-	GossipContactList *contact_list;
-	MissionControl    *mc;
+	GossipContactListView  *list_view;
+	GossipContactListStore *list_store;
+	MissionControl         *mc;
+	GossipChatroomManager  *chatroom_manager;
 
 	/* Main widgets */
-	GtkWidget         *window;
-	GtkWidget         *main_vbox;
+	GtkWidget              *window;
+	GtkWidget              *main_vbox;
 
 	/* Tooltips for all widgets */
-	GtkTooltips       *tooltips;
+	GtkTooltips            *tooltips;
 
 	/* Menu widgets */
-	GtkWidget         *room;
-	GtkWidget         *room_menu;
-	GtkWidget         *room_sep;
-	GtkWidget         *room_join_favorites;
-	GtkWidget         *edit_context;
-	GtkWidget         *edit_context_separator;
+	GtkWidget              *room;
+	GtkWidget              *room_menu;
+	GtkWidget              *room_sep;
+	GtkWidget              *room_join_favorites;
+	GtkWidget              *edit_context;
+	GtkWidget              *edit_context_separator;
 
 	/* Throbber */
-	GtkWidget         *throbber;
+	GtkWidget              *throbber;
 
-	/* Widgets that are enabled when we're connected/disconnected */
-	GList             *widgets_connected;
-	GList             *widgets_disconnected;
+	/* Widgets that are enabled when there is... */
+	GList                  *widgets_connected;	/* ... connected accounts */
+	GList                  *widgets_disconnected;	/* ... disconnected accounts */
 
 	/* Status popup */
-	GtkWidget         *presence_toolbar;
-	GtkWidget         *presence_chooser;
+	GtkWidget              *presence_toolbar;
+	GtkWidget              *presence_chooser;
 
 	/* Misc */
-	guint              size_timeout_id;
+	guint                   size_timeout_id;
 } EmpathyMainWindow;
 
-static void     main_window_destroy_cb                     (GtkWidget           *widget,
-							    EmpathyMainWindow   *window);
-static void     main_window_favorite_chatroom_menu_setup   (void);
-static void     main_window_chat_quit_cb                   (GtkWidget           *widget,
-							    EmpathyMainWindow   *window);
-static void     main_window_chat_new_message_cb            (GtkWidget           *widget,
-							    EmpathyMainWindow   *window);
-static void     main_window_chat_history_cb                (GtkWidget           *widget,
-							    EmpathyMainWindow   *window);
-static void     main_window_room_join_new_cb               (GtkWidget           *widget,
-							    EmpathyMainWindow   *window);
-static void     main_window_room_join_favorites_cb         (GtkWidget           *widget,
-							    EmpathyMainWindow   *window);
-static void     main_window_room_manage_favorites_cb       (GtkWidget           *widget,
-							    EmpathyMainWindow   *window);
-static void     main_window_chat_add_contact_cb            (GtkWidget           *widget,
-							    EmpathyMainWindow   *window);
-static void     main_window_chat_show_offline_cb           (GtkCheckMenuItem    *item,
-							    EmpathyMainWindow   *window);
-static gboolean main_window_edit_button_press_event_cb     (GtkWidget           *widget,
-							    GdkEventButton      *event,
-							    EmpathyMainWindow   *window);
-static void     main_window_edit_accounts_cb               (GtkWidget           *widget,
-							    EmpathyMainWindow   *window);
-static void     main_window_edit_personal_information_cb   (GtkWidget           *widget,
-							    EmpathyMainWindow   *window);
-static void     main_window_edit_preferences_cb            (GtkWidget           *widget,
-							    EmpathyMainWindow   *window);
-static void     main_window_help_about_cb                  (GtkWidget           *widget,
-							    EmpathyMainWindow   *window);
-static void     main_window_help_contents_cb               (GtkWidget           *widget,
-							    EmpathyMainWindow   *window);
-static gboolean main_window_throbber_button_press_event_cb (GtkWidget           *throbber_ebox,
-							    GdkEventButton      *event,
-							    gpointer             user_data);
-static void     main_window_accels_load                    (void);
-static void     main_window_accels_save                    (void);
-static void     main_window_connection_items_setup         (EmpathyMainWindow   *window,
-							    GladeXML            *glade);
-//static void     main_window_connection_items_update        (void);
-static void     main_window_presence_changed_cb            (DBusGProxy          *proxy,
-							    McPresence           state,
-							    EmpathyMainWindow   *window);
-static void     main_window_presence_chooser_changed_cb    (GtkWidget           *chooser,
-							    McPresence           state,
-							    const gchar         *status,
-							    EmpathyMainWindow   *window);
-static gboolean main_window_configure_event_timeout_cb     (EmpathyMainWindow   *window);
-static gboolean main_window_configure_event_cb             (GtkWidget           *widget,
-							    GdkEventConfigure   *event,
-							    EmpathyMainWindow   *window);
-static void     main_window_notify_show_offline_cb         (GossipConf          *conf,
-							    const gchar         *key,
-							    gpointer             check_menu_item);
-static void     main_window_notify_show_avatars_cb         (GossipConf          *conf,
-							    const gchar         *key,
-							    EmpathyMainWindow   *window);
-static void     main_window_notify_compact_contact_list_cb (GossipConf          *conf,
-							    const gchar         *key,
-							    EmpathyMainWindow   *window);
+static void     main_window_destroy_cb                         (GtkWidget                       *widget,
+								EmpathyMainWindow               *window);
+static void     main_window_favorite_chatroom_menu_setup       (EmpathyMainWindow               *window);
+static void     main_window_favorite_chatroom_menu_added_cb    (GossipChatroomManager           *manager,
+								GossipChatroom                  *chatroom,
+								EmpathyMainWindow               *window);
+static void     main_window_favorite_chatroom_menu_removed_cb  (GossipChatroomManager           *manager,
+								GossipChatroom                  *chatroom,
+								EmpathyMainWindow               *window);
+static void     main_window_favorite_chatroom_menu_activate_cb (GtkMenuItem                     *menu_item,
+								GossipChatroom                  *chatroom);
+static void     main_window_favorite_chatroom_menu_update      (EmpathyMainWindow               *window);
+static void     main_window_favorite_chatroom_menu_add         (EmpathyMainWindow               *window,
+								GossipChatroom                  *chatroom);
+static void     main_window_favorite_chatroom_join             (GossipChatroom                  *chatroom);
+static void     main_window_chat_quit_cb                       (GtkWidget                       *widget,
+								EmpathyMainWindow               *window);
+static void     main_window_chat_new_message_cb                (GtkWidget                       *widget,
+								EmpathyMainWindow               *window);
+static void     main_window_chat_history_cb                    (GtkWidget                       *widget,
+								EmpathyMainWindow               *window);
+static void     main_window_room_join_new_cb                   (GtkWidget                       *widget,
+								EmpathyMainWindow               *window);
+static void     main_window_room_join_favorites_cb             (GtkWidget                       *widget,
+								EmpathyMainWindow               *window);
+static void     main_window_room_manage_favorites_cb           (GtkWidget                       *widget,
+								EmpathyMainWindow               *window);
+static void     main_window_chat_add_contact_cb                (GtkWidget                       *widget,
+								EmpathyMainWindow               *window);
+static void     main_window_chat_show_offline_cb               (GtkCheckMenuItem                *item,
+								EmpathyMainWindow               *window);
+static gboolean main_window_edit_button_press_event_cb         (GtkWidget                       *widget,
+								GdkEventButton                  *event,
+								EmpathyMainWindow               *window);
+static void     main_window_edit_accounts_cb                   (GtkWidget                       *widget,
+								EmpathyMainWindow               *window);
+static void     main_window_edit_personal_information_cb       (GtkWidget                       *widget,
+								EmpathyMainWindow               *window);
+static void     main_window_edit_preferences_cb                (GtkWidget                       *widget,
+								EmpathyMainWindow               *window);
+static void     main_window_help_about_cb                      (GtkWidget                       *widget,
+								EmpathyMainWindow               *window);
+static void     main_window_help_contents_cb                   (GtkWidget                       *widget,
+								EmpathyMainWindow               *window);
+static gboolean main_window_throbber_button_press_event_cb     (GtkWidget                       *throbber_ebox,
+								GdkEventButton                  *event,
+								EmpathyMainWindow               *window);
+static void     main_window_status_changed_cb                  (MissionControl                  *mc,
+								TelepathyConnectionStatus        status,
+								McPresence                       presence,
+								TelepathyConnectionStatusReason  reason,
+								const gchar                     *unique_name,
+								EmpathyMainWindow               *window);
+static void     main_window_update_status                      (EmpathyMainWindow               *window);
+static void     main_window_accels_load                        (void);
+static void     main_window_accels_save                        (void);
+static void     main_window_connection_items_setup             (EmpathyMainWindow               *window,
+								GladeXML                        *glade);
+static gboolean main_window_configure_event_timeout_cb         (EmpathyMainWindow               *window);
+static gboolean main_window_configure_event_cb                 (GtkWidget                       *widget,
+								GdkEventConfigure               *event,
+								EmpathyMainWindow               *window);
+static void     main_window_notify_show_offline_cb             (GossipConf                      *conf,
+								const gchar                     *key,
+								gpointer                         check_menu_item);
+static void     main_window_notify_show_avatars_cb             (GossipConf                      *conf,
+								const gchar                     *key,
+								EmpathyMainWindow               *window);
+static void     main_window_notify_compact_contact_list_cb     (GossipConf                      *conf,
+								const gchar                     *key,
+								EmpathyMainWindow               *window);
+static void     main_window_notify_sort_criterium_cb           (GossipConf                      *conf,
+								const gchar                     *key,
+								EmpathyMainWindow               *window);
 
 GtkWidget *
 empathy_main_window_show (void)
 {
 	static EmpathyMainWindow *window = NULL;
+	EmpathyContactList       *list_iface;
 	GladeXML                 *glade;
 	GossipConf               *conf;
 	GtkWidget                *sw;
@@ -164,7 +184,6 @@ empathy_main_window_show (void)
 	GtkWidget                *ebox;
 	GtkToolItem              *item;
 	gchar                    *str;
-	McPresence                state;
 	gboolean                  show_offline;
 	gboolean                  show_avatars;
 	gboolean                  compact_contact_list;
@@ -218,34 +237,20 @@ empathy_main_window_show (void)
 	g_object_unref (glade);
 
 	window->tooltips = g_object_ref_sink (gtk_tooltips_new ());
+	window->mc = gossip_mission_control_new ();
+	dbus_g_proxy_connect_signal (DBUS_G_PROXY (window->mc), "AccountStatusChanged",
+				     G_CALLBACK (main_window_status_changed_cb),
+				     window, NULL);
 
 	/* Set up menu */
-	main_window_favorite_chatroom_menu_setup ();
+	main_window_favorite_chatroom_menu_setup (window);
 
 	gtk_widget_hide (window->edit_context);
 	gtk_widget_hide (window->edit_context_separator);
 
-	/* Set up presence chooser
-	 * FIXME: Update status message not yet supported by MC 
-	 */
-	window->mc = mission_control_new (tp_get_bus ());
+	/* Set up presence chooser */
 	window->presence_chooser = gossip_presence_chooser_new ();
 	gtk_widget_show (window->presence_chooser);
-	gossip_presence_chooser_set_flash_interval (GOSSIP_PRESENCE_CHOOSER (window->presence_chooser),
-						    FLASH_TIMEOUT);
-	state = mission_control_get_presence (window->mc, NULL);
-	gossip_presence_chooser_set_state (GOSSIP_PRESENCE_CHOOSER (window->presence_chooser),
-					   state);
-	dbus_g_proxy_connect_signal (DBUS_G_PROXY (window->mc),
-				     "PresenceStatusRequested",
-				     G_CALLBACK (main_window_presence_changed_cb),
-				     window, NULL);
-	g_signal_connect (window->presence_chooser,
-			  "changed",
-			  G_CALLBACK (main_window_presence_chooser_changed_cb),
-			  window);
-
-
 	item = gtk_tool_item_new ();
 	gtk_widget_show (GTK_WIDGET (item));
 	gtk_container_add (GTK_CONTAINER (item), window->presence_chooser);
@@ -253,9 +258,6 @@ empathy_main_window_show (void)
 	gtk_tool_item_set_expand (item, TRUE);
 	gtk_toolbar_insert (GTK_TOOLBAR (window->presence_toolbar), item, -1);
 
-	window->widgets_connected = g_list_prepend (window->widgets_connected,
-						  window->presence_chooser);
-
 	/* Set up the throbber */
 	ebox = gtk_event_box_new ();
 	gtk_event_box_set_visible_window (GTK_EVENT_BOX (ebox), FALSE);
@@ -277,14 +279,20 @@ empathy_main_window_show (void)
 	g_signal_connect (ebox,
 			  "button-press-event",
 			  G_CALLBACK (main_window_throbber_button_press_event_cb),
-			  NULL);
+			  window);
 
 	/* Set up contact list. */
 	gossip_status_presets_get_all ();
-	window->contact_list = gossip_contact_list_new ();
-	gtk_widget_show (GTK_WIDGET (window->contact_list));
+
+	list_iface = EMPATHY_CONTACT_LIST (empathy_contact_manager_new ());
+	empathy_contact_list_setup (list_iface);
+	window->list_store = gossip_contact_list_store_new (list_iface);
+	window->list_view = gossip_contact_list_view_new (window->list_store);
+	g_object_unref (list_iface);
+
+	gtk_widget_show (GTK_WIDGET (window->list_view));
 	gtk_container_add (GTK_CONTAINER (sw),
-			   GTK_WIDGET (window->contact_list));
+			   GTK_WIDGET (window->list_view));
 
 	/* Load user-defined accelerators. */
 	main_window_accels_load ();
@@ -308,8 +316,9 @@ empathy_main_window_show (void)
 		gtk_window_move (GTK_WINDOW (window->window), x, y);
 	}
 
-	/* Get preferences */
 	conf = gossip_conf_get ();
+	
+	/* Show offline ? */
 	gossip_conf_get_bool (conf,
 			      GOSSIP_PREFS_CONTACTS_SHOW_OFFLINE,
 			      &show_offline);
@@ -321,6 +330,7 @@ empathy_main_window_show (void)
 	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (show_offline_widget),
 					show_offline);
 
+	/* Show avatars ? */
 	gossip_conf_get_bool (conf,
 			      GOSSIP_PREFS_UI_SHOW_AVATARS,
 			      &show_avatars);
@@ -328,6 +338,9 @@ empathy_main_window_show (void)
 				GOSSIP_PREFS_UI_SHOW_AVATARS,
 				(GossipConfNotifyFunc) main_window_notify_show_avatars_cb,
 				window);
+	gossip_contact_list_store_set_show_avatars (window->list_store, show_avatars);
+
+	/* Is compact ? */
 	gossip_conf_get_bool (conf,
 			      GOSSIP_PREFS_UI_COMPACT_CONTACT_LIST,
 			      &compact_contact_list);
@@ -335,13 +348,18 @@ empathy_main_window_show (void)
 				GOSSIP_PREFS_UI_COMPACT_CONTACT_LIST,
 				(GossipConfNotifyFunc) main_window_notify_compact_contact_list_cb,
 				window);
+	gossip_contact_list_store_set_is_compact (window->list_store, compact_contact_list);
 
-	g_object_set (window->contact_list,
-		      "show-avatars", show_avatars,
-		      "is-compact", compact_contact_list,
-		      NULL);
+	/* Sort criterium */
+	gossip_conf_notify_add (conf,
+				GOSSIP_PREFS_CONTACTS_SORT_CRITERIUM,
+				(GossipConfNotifyFunc) main_window_notify_sort_criterium_cb,
+				window);
+	main_window_notify_sort_criterium_cb (conf,
+					      GOSSIP_PREFS_CONTACTS_SORT_CRITERIUM,
+					      window);
 
-	gtk_widget_show (window->window);
+	main_window_update_status (window);
 
 	return window->window;
 }
@@ -353,6 +371,10 @@ main_window_destroy_cb (GtkWidget         *widget,
 	/* Save user-defined accelerators. */
 	main_window_accels_save ();
 
+	dbus_g_proxy_disconnect_signal (DBUS_G_PROXY (window->mc), "AccountStatusChanged",
+					G_CALLBACK (main_window_status_changed_cb),
+					window);
+
 	if (window->size_timeout_id) {
 		g_source_remove (window->size_timeout_id);
 	}
@@ -362,20 +384,141 @@ main_window_destroy_cb (GtkWidget         *widget,
 
 	g_object_unref (window->tooltips);
 	g_object_unref (window->mc);
+	g_object_unref (window->list_store);
 
 	g_free (window);
 }
 
 static void
-main_window_favorite_chatroom_menu_setup (void)
+main_window_favorite_chatroom_menu_setup (EmpathyMainWindow *window)
 {
+	GList *chatrooms, *l;
+
+	window->chatroom_manager = gossip_chatroom_manager_new ();
+	chatrooms = gossip_chatroom_manager_get_chatrooms (window->chatroom_manager, NULL);
+	window->room_menu = gtk_menu_item_get_submenu (GTK_MENU_ITEM (window->room));
+
+	for (l = chatrooms; l; l = l->next) {
+		main_window_favorite_chatroom_menu_add (window, l->data);
+	}
+
+	if (!chatrooms) {
+		gtk_widget_hide (window->room_sep);
+	}
+
+	gtk_widget_set_sensitive (window->room_join_favorites, chatrooms != NULL);
+
+	g_signal_connect (window->chatroom_manager, "chatroom-added",
+			  G_CALLBACK (main_window_favorite_chatroom_menu_added_cb),
+			  window);
+	g_signal_connect (window->chatroom_manager, "chatroom-removed",
+			  G_CALLBACK (main_window_favorite_chatroom_menu_removed_cb),
+			  window);
+
+	g_list_free (chatrooms);
+}
+
+static void
+main_window_favorite_chatroom_menu_added_cb (GossipChatroomManager *manager,
+					     GossipChatroom        *chatroom,
+					     EmpathyMainWindow     *window)
+{
+	main_window_favorite_chatroom_menu_add (window, chatroom);
+	gtk_widget_show (window->room_sep);
+	gtk_widget_set_sensitive (window->room_join_favorites, TRUE);
+}
+
+static void
+main_window_favorite_chatroom_menu_removed_cb (GossipChatroomManager *manager,
+					       GossipChatroom        *chatroom,
+					       EmpathyMainWindow     *window)
+{
+	GtkWidget *menu_item;
+
+	menu_item = g_object_get_data (G_OBJECT (chatroom), "menu_item");
+
+	g_object_set_data (G_OBJECT (chatroom), "menu_item", NULL);
+	gtk_widget_destroy (menu_item);
+
+	main_window_favorite_chatroom_menu_update (window);
+}
+
+static void
+main_window_favorite_chatroom_menu_activate_cb (GtkMenuItem    *menu_item,
+						GossipChatroom *chatroom)
+{
+	main_window_favorite_chatroom_join (chatroom);
+}
+
+static void
+main_window_favorite_chatroom_menu_update (EmpathyMainWindow *window)
+{
+	GList *chatrooms;
+
+	chatrooms = gossip_chatroom_manager_get_chatrooms (window->chatroom_manager, NULL);
+
+	if (chatrooms) {
+		gtk_widget_show (window->room_sep);
+	} else {
+		gtk_widget_hide (window->room_sep);
+	}
+
+	gtk_widget_set_sensitive (window->room_join_favorites, chatrooms != NULL);
+	g_list_free (chatrooms);
+}
+
+static void
+main_window_favorite_chatroom_menu_add (EmpathyMainWindow *window,
+					GossipChatroom    *chatroom)
+{
+	GtkWidget   *menu_item;
+	const gchar *name;
+
+	if (g_object_get_data (G_OBJECT (chatroom), "menu_item")) {
+		return;
+	}
+
+	name = gossip_chatroom_get_name (chatroom);
+	menu_item = gtk_menu_item_new_with_label (name);
+
+	g_object_set_data (G_OBJECT (chatroom), "menu_item", menu_item);
+	g_signal_connect (menu_item, "activate",
+			  G_CALLBACK (main_window_favorite_chatroom_menu_activate_cb),
+			  chatroom);
+
+	gtk_menu_shell_insert (GTK_MENU_SHELL (window->room_menu),
+			       menu_item, 3);
+
+	gtk_widget_show (menu_item);
+}
+
+static void
+main_window_favorite_chatroom_join (GossipChatroom *chatroom)
+{
+	MissionControl *mc;
+	McAccount      *account;
+	const gchar    *room;
+
+	mc = gossip_mission_control_new ();
+	account = gossip_chatroom_get_account (chatroom);
+	room = gossip_chatroom_get_room (chatroom);
+
+	gossip_debug (DEBUG_DOMAIN, "Requesting channel for '%s'", room);
+
+	mission_control_request_channel_with_string_handle (mc,
+							    account,
+							    TP_IFACE_CHANNEL_TYPE_TEXT,
+							    room,
+							    TP_HANDLE_TYPE_ROOM,
+							    NULL, NULL);	
+	g_object_unref (mc);
 }
 
 static void
 main_window_chat_quit_cb (GtkWidget         *widget,
 			  EmpathyMainWindow *window)
 {
-	gtk_widget_destroy (window->window);
+	gtk_main_quit ();
 }
 
 static void
@@ -396,21 +539,27 @@ static void
 main_window_room_join_new_cb (GtkWidget         *widget,
 			      EmpathyMainWindow *window)
 {
-	//gossip_new_chatroom_dialog_show (GTK_WINDOW (window->window));
+	gossip_new_chatroom_dialog_show (GTK_WINDOW (window->window));
 }
 
 static void
 main_window_room_join_favorites_cb (GtkWidget         *widget,
 				    EmpathyMainWindow *window)
 {
-	//gossip_session_chatroom_join_favorites (window->session);
+	GList *chatrooms, *l;
+
+	chatrooms = gossip_chatroom_manager_get_chatrooms (window->chatroom_manager, NULL);
+	for (l = chatrooms; l; l = l->next) {
+		main_window_favorite_chatroom_join (l->data);
+	}
+	g_list_free (chatrooms);
 }
 
 static void
 main_window_room_manage_favorites_cb (GtkWidget         *widget,
 				      EmpathyMainWindow *window)
 {
-	//gossip_chatrooms_window_show (NULL, FALSE);
+	gossip_chatrooms_window_show (GTK_WINDOW (window->window));
 }
 
 static void
@@ -434,7 +583,7 @@ main_window_chat_show_offline_cb (GtkCheckMenuItem  *item,
 
 	/* Turn off sound just while we alter the contact list. */
 	// FIXME: gossip_sound_set_enabled (FALSE);
-	g_object_set (window->contact_list, "show_offline", current, NULL);
+	gossip_contact_list_store_set_show_offline (window->list_store, current);
 	//gossip_sound_set_enabled (TRUE);
 }
 
@@ -450,7 +599,7 @@ main_window_edit_button_press_event_cb (GtkWidget         *widget,
 		return FALSE;
 	}
 
-	group = gossip_contact_list_get_selected_group (window->contact_list);
+	group = gossip_contact_list_view_get_selected_group (window->list_view);
 	if (group) {
 		GtkMenuItem *item;
 		GtkWidget   *label;
@@ -463,7 +612,7 @@ main_window_edit_button_press_event_cb (GtkWidget         *widget,
 		gtk_widget_show (window->edit_context);
 		gtk_widget_show (window->edit_context_separator);
 
-		submenu = gossip_contact_list_get_group_menu (window->contact_list);
+		submenu = gossip_contact_list_view_get_group_menu (window->list_view);
 		gtk_menu_item_set_submenu (item, submenu);
 
 		g_free (group);
@@ -471,7 +620,7 @@ main_window_edit_button_press_event_cb (GtkWidget         *widget,
 		return FALSE;
 	}
 
-	contact = gossip_contact_list_get_selected (window->contact_list);
+	contact = gossip_contact_list_view_get_selected (window->list_view);
 	if (contact) {
 		GtkMenuItem *item;
 		GtkWidget   *label;
@@ -484,8 +633,8 @@ main_window_edit_button_press_event_cb (GtkWidget         *widget,
 		gtk_widget_show (window->edit_context);
 		gtk_widget_show (window->edit_context_separator);
 
-		submenu = gossip_contact_list_get_contact_menu (window->contact_list,
-								contact);
+		submenu = gossip_contact_list_view_get_contact_menu (window->list_view,
+								     contact);
 		gtk_menu_item_set_submenu (item, submenu);
 
 		g_object_unref (contact);
@@ -503,7 +652,7 @@ static void
 main_window_edit_accounts_cb (GtkWidget         *widget,
 			      EmpathyMainWindow *window)
 {
-	gossip_accounts_dialog_show ();
+	gossip_accounts_dialog_show (GTK_WINDOW (window->window));
 }
 
 static void
@@ -517,14 +666,14 @@ static void
 main_window_edit_preferences_cb (GtkWidget         *widget,
 				 EmpathyMainWindow *window)
 {
-	gossip_preferences_show ();
+	gossip_preferences_show (GTK_WINDOW (window->window));
 }
 
 static void
 main_window_help_about_cb (GtkWidget         *widget,
 			   EmpathyMainWindow *window)
 {
-	//gossip_about_dialog_new (GTK_WINDOW (window->window));
+	gossip_about_dialog_new (GTK_WINDOW (window->window));
 }
 
 static void
@@ -535,122 +684,79 @@ main_window_help_contents_cb (GtkWidget         *widget,
 }
 
 static gboolean
-main_window_throbber_button_press_event_cb (GtkWidget      *throbber_ebox,
-					    GdkEventButton *event,
-					    gpointer        user_data)
+main_window_throbber_button_press_event_cb (GtkWidget         *throbber_ebox,
+					    GdkEventButton    *event,
+					    EmpathyMainWindow *window)
 {
 	if (event->type != GDK_BUTTON_PRESS ||
 	    event->button != 1) {
 		return FALSE;
 	}
 
-	gossip_accounts_dialog_show ();
+	gossip_accounts_dialog_show (GTK_WINDOW (window->window));
 
 	return FALSE;
 }
-#if 0
+
 static void
-main_window_session_protocol_connecting_cb (GossipSession  *session,
-				    GossipAccount  *account,
-				    GossipProtocol *protocol,
-				    gpointer        user_data)
+main_window_status_changed_cb (MissionControl                  *mc,
+			       TelepathyConnectionStatus        status,
+			       McPresence                       presence,
+			       TelepathyConnectionStatusReason  reason,
+			       const gchar                     *unique_name,
+			       EmpathyMainWindow               *window)
 {
-	GossipAppPriv *priv;
-	const gchar   *name;
-
-	priv = GET_PRIV (app);
-
-	name = gossip_account_get_name (account);
-	gossip_debug (DEBUG_DOMAIN, "Connecting account:'%s'", name);
-
-	ephy_spinner_start (EPHY_SPINNER (window->throbber));
+	main_window_update_status (window);
 }
 
 static void
-main_window_session_protocol_connected_cb (GossipSession  *session,
-				   GossipAccount  *account,
-				   GossipProtocol *protocol,
-				   gpointer        user_data)
+main_window_update_status (EmpathyMainWindow *window)
 {
-	GossipAppPriv *priv;
-	gboolean       connecting;
-	const gchar   *name;
+	GList *accounts, *l;
+	guint  connected = 0;
+	guint  connecting = 0;
+	guint  disconnected = 0;
 
-	priv = GET_PRIV (app);
+	/* Count number of connected/connecting/disconnected accounts */
+	accounts = mc_accounts_list ();	
+	for (l = accounts; l; l = l->next) {
+		McAccount *account;
+		guint      status;
 
-	name = gossip_account_get_name (account);
-	gossip_debug (DEBUG_DOMAIN, "Connected account:'%s'", name);
+		account = l->data;
 
-	gossip_session_count_accounts (window->session,
-				       NULL,
-				       &connecting,
-				       NULL);
+		status = mission_control_get_connection_status (window->mc,
+								account,
+								NULL);
 
-	if (connecting < 1) {
+		if (status == 0) {
+			connected++;
+		} else if (status == 1) {
+			connecting++;
+		} else if (status == 2) {
+			disconnected++;
+		}
+
+		g_object_unref (account);
+	}
+	g_list_free (accounts);
+
+	/* Update the spinner state */
+	if (connecting > 0) {
+		ephy_spinner_start (EPHY_SPINNER (window->throbber));
+	} else {
 		ephy_spinner_stop (EPHY_SPINNER (window->throbber));
 	}
 
-	g_hash_table_remove (window->errors, account);
-	g_hash_table_remove (window->reconnects, account);
-
-	app_connection_items_update ();
-	app_favorite_chatroom_menu_update ();
-
-	/* Use saved presence */
-	gossip_app_set_presence (gossip_status_presets_get_default_state (),
-				 gossip_status_presets_get_default_status());
-
-	app_presence_updated ();
-}
-
-static void
-main_window_session_protocol_disconnected_cb (GossipSession  *session,
-				      GossipAccount  *account,
-				      GossipProtocol *protocol,
-				      gint            reason,
-				      gpointer        user_data)
-{
-	GossipAppPriv *priv;
-	gboolean       connecting;
-	gboolean       should_reconnect;
-	const gchar   *name;
-
-	priv = GET_PRIV (app);
-
-	name = gossip_account_get_name (account);
-	gossip_debug (DEBUG_DOMAIN, "Disconnected account:'%s'", name);
-
-	gossip_session_count_accounts (window->session,
-				       NULL,
-				       &connecting,
-				       NULL);
-
-	if (connecting < 1) {
-		ephy_spinner_stop (EPHY_SPINNER (window->throbber));
+	/* Update widgets sensibility */
+	for (l = window->widgets_connected; l; l = l->next) {
+		gtk_widget_set_sensitive (l->data, (connected > 0));
 	}
 
-	app_connection_items_update ();
-	app_favorite_chatroom_menu_update ();
-	app_presence_updated ();
-
-	should_reconnect = reason != GOSSIP_PROTOCOL_DISCONNECT_ASKED;
-
-
-	should_reconnect &= !g_hash_table_lookup (window->reconnects, account);
-
-	if (should_reconnect) {
-		guint id;
-
-		/* Unexpected disconnection, try to reconnect */
-		id = g_timeout_add (RETRY_CONNECT_TIMEOUT * 1000,
-				    (GSourceFunc) app_reconnect_cb,
-				    account);
-		g_hash_table_insert (window->reconnects,
-				     g_object_ref (account),
-				     &id);
+	for (l = window->widgets_disconnected; l; l = l->next) {
+		gtk_widget_set_sensitive (l->data, (disconnected > 0));
 	}
 }
-#endif
 
 /*
  * Accels
@@ -694,14 +800,12 @@ main_window_connection_items_setup (EmpathyMainWindow *window,
 	GtkWidget     *w;
 	gint           i;
 	const gchar *widgets_connected[] = {
-		"chat_disconnect",
 		"room",
 		"chat_new_message",
 		"chat_add_contact",
 		"edit_personal_information"
 	};
 	const gchar *widgets_disconnected[] = {
-		"chat_connect"
 	};
 
 	for (i = 0, list = NULL; i < G_N_ELEMENTS (widgets_connected); i++) {
@@ -719,55 +823,6 @@ main_window_connection_items_setup (EmpathyMainWindow *window,
 	window->widgets_disconnected = list;
 }
 
-#if 0
-FIXME:
-static void
-main_window_connection_items_update (void)
-{
-	GList         *l;
-	guint          connected = 0;
-	guint          disconnected = 0;
-
-	/* Get account count for:
-	 *  - connected and disabled,
-	 *  - connected and enabled
-	 *  - disabled and enabled
-	 */
-	gossip_session_count_accounts (window->session,
-				       &connected,
-				       NULL,
-				       &disconnected);
-
-	for (l = window->widgets_connected; l; l = l->next) {
-		gtk_widget_set_sensitive (l->data, (connected > 0));
-	}
-
-	for (l = window->widgets_disconnected; l; l = l->next) {
-		gtk_widget_set_sensitive (l->data, (disconnected > 0));
-	}
-}
-#endif
-
-static void
-main_window_presence_changed_cb (DBusGProxy        *proxy,
-				 McPresence         state,
-				 EmpathyMainWindow *window)
-{
-	gossip_debug (DEBUG_DOMAIN, "presence changed to %d", state);
-	gossip_presence_chooser_set_state (GOSSIP_PRESENCE_CHOOSER (window->presence_chooser),
-					   state);
-}
-
-static void
-main_window_presence_chooser_changed_cb (GtkWidget         *chooser,
-					 McPresence         state,
-					 const gchar       *status,
-					 EmpathyMainWindow *window)
-{
-	gossip_status_presets_set_default (state, status);
-	mission_control_set_presence (window->mc, state, status, NULL, NULL);
-}
-
 static gboolean
 main_window_configure_event_timeout_cb (EmpathyMainWindow *window)
 {
@@ -820,8 +875,8 @@ main_window_notify_show_avatars_cb (GossipConf        *conf,
 	gboolean show_avatars;
 
 	if (gossip_conf_get_bool (conf, key, &show_avatars)) {
-		gossip_contact_list_set_show_avatars (window->contact_list,
-						      show_avatars);
+		gossip_contact_list_store_set_show_avatars (window->list_store,
+							    show_avatars);
 	}
 }
 
@@ -833,8 +888,32 @@ main_window_notify_compact_contact_list_cb (GossipConf        *conf,
 	gboolean compact_contact_list;
 
 	if (gossip_conf_get_bool (conf, key, &compact_contact_list)) {
-		gossip_contact_list_set_is_compact (window->contact_list,
-						    compact_contact_list);
+		gossip_contact_list_store_set_is_compact (window->list_store,
+							  compact_contact_list);
+	}
+}
+
+static void
+main_window_notify_sort_criterium_cb (GossipConf        *conf,
+				      const gchar       *key,
+				      EmpathyMainWindow *window)
+{
+	gchar *str = NULL;
+
+	if (gossip_conf_get_string (conf, key, &str)) {
+		GType       type;
+		GEnumClass *enum_class;
+		GEnumValue *enum_value;
+
+		type = gossip_contact_list_store_sort_get_type ();
+		enum_class = G_ENUM_CLASS (g_type_class_peek (type));
+		enum_value = g_enum_get_value_by_nick (enum_class, str);
+		g_free (str);
+
+		if (enum_value) {
+			gossip_contact_list_store_set_sort_criterium (window->list_store, 
+								      enum_value->value);
+		}
 	}
 }
 
